//genesis
/**********************************************************************
** This demo is      
**                  copyright (C) 1998 Upinder S. Bhalla.
** It is made available under the terms of the GNU General Public License. 
** See the file COPYRIGHT for the full notice.
**********************************************************************/

// This file sets up the cell model. It is based on Dave Beeman's
// implementation of the Traub91 model, and is derived from the
// working file scell29.g with the interface spruced up and some
// of the manually entered parameters predefined.
// This demo takes a very long time to run (several hours on a 
// PII-266). However, most of this time is spent working out the
// Ca decay time course, which is not particularly exciting. The
// stimulus response itself should be done within the first hour.
// Note that all concentrations are in mM (millimolar). The units
// used in the paper are micromolar.
// The waveform used in the paper is abstr2_ca.gmn, which was
// abstracted from the simulated waveform by hand to produce a much
// more compact file. The three Ca transients in the stimulus are so
// far apart that the Ca is assumed to return to baseline by the
// time the next stimulus comes along. The abstracted waveform is also
// scaled to micromolar. 
// The simulation was found to be very insensitive to the exact Ca
// levels or waveforms, so the approximations in the
// abstracted file were acceptable.

/**********************************************************************
**
**	1	Including script files for prototype functions
**
**********************************************************************/

floatformat %0.5g
xcolorscale hot

/* file for standard compartments */
include compartments
/* file for 1991 Traub model channels */
include traub91proto.g
/* file for Erik's spine model channels */
// This proto has been updated to use nernst potls for Ca.
include mar7b_96_proto

/**********************************************************************
**
**	2	Invoking functions to make prototypes in the /library element
**
**********************************************************************/

create neutral /library
disable /library
/* To ensure that all subsequent elements are made in the library */
	ce /library

	make_cylind_compartment		/* makes "compartment" */
	make_cylind_symcompartment	/* makes "symcompartment" */

	make_the_spine_prototypes

/* Assign some constants to override those used in traub91proto.g */
EREST_ACT = -0.06       // resting membrane potential (volts)
float ENA = 0.115 + EREST_ACT // 0.055  when EREST_ACT = -0.060
float EK = -0.015 + EREST_ACT // -0.075
float ECA = 0.140 + EREST_ACT // 0.080
float NMDA_AMPL	= 0.45
float non_NMDA_AMPL	= 25
int NPULSES = 100 // 1 sec
float PULSEFREQ = 100.0
float TET_SPACING = 600.0
float PRE_TET_TIME = 0.1
float POST_TET_TIME = 50
int NTETANI = 1
str FILENAME = "/tmp/celldemo"

make_Na
make_Ca
make_K_DR
make_K_AHP
make_K_C
make_K_A
make_Ca_conc

ce /		/* return to the root element */

float accurate_dt = 2e-7  // 0.2 usec
float intertrial_dt = 1e-6  // 2 usec
float plot_dt = 2e-3  // 2 msec
float cell_dt = 1000e-3  // 1 sec
float currtime_dt = 0.005  // 0.005 sec


setclock 0 {accurate_dt}
setclock 1 {plot_dt}
setclock 2 {cell_dt}
setclock 3 {currtime_dt}

create neutral /cell
readcell spinyca1.p /cell/CA1
str name

create hsolve /cell/CA1/solve
setfield /cell/CA1/solve \
	path /cell/CA1/##[][CLASS=membrane] \
	comptmode 1 \
	chanmode 3 \
	useconcen 1




create xform /form [0,0,500,400]
create xdraw /form/draw [0,0,100%,100%] \
	-xmin -5e-6 -xmax 5e-6 \
	-ymin -5e-6 -ymax 5e-6 \
	-zmin -5e-6 -zmax 5e-6 \
	-transform o

create xcell /form/draw/cell
useclock /form/draw/cell 2
disable /form

setfield /form/draw/cell \
	colmin -0.08 \
	colmax 0.04 \
	diarange 0.2 \
	diamin 0	\
	diamax 1e-5 \
	field Vm \
	path /cell/CA1/##[CLASS=membrane]

call /form/draw/cell RESET

function update_time
	setfield /control/currtime value {getstat -time}
end

function do_step
	step {getfield /control/runtime value} -t -b
end

function save_plot(plot,suffix)
	str plot,suffix
        str filename = {getfield /control/filename value}

        echo >> {filename}.{suffix}
        echo /newplot >> {filename}.{suffix}
        echo /plotname {filename} >> {filename}.{suffix}
        tab2file {filename}.{suffix} {plot} xpts -table2 ypts \
                -nentries {{getfield {plot} npts} - 1}
end

function do_stim
	int ntetani = {getfield /control/ntetani value}
	float tet_spacing = {getfield /control/tet_spacing value}
	float pulsefreq = {getfield /control/pulsefreq value}
	float pulsewidth = {getfield /control/pulsewidth value}
	float inter_pulse_width = 1.0/pulsefreq - pulsewidth
	int i,j
	int npulses = {getfield /control/npulses value}
	float tet_duration = npulses/pulsefreq
	float pre_tet_time = {getfield /control/pre_tet_time value}
	float post_tet_time = {getfield /control/post_tet_time value}

	echo npulses = {npulses}, inter_pulse_width = {inter_pulse_width}
	setclock 0 {intertrial_dt}

	if (inter_pulse_width < 0)
		echo Warning: pulsefreq {pulsefreq} not possible with
		echo pulsewidth = {pulsewidth}
		return
	end
	step {pre_tet_time} -t

	if (ntetani > 0)
		for (j = 0; j < ntetani; j = j + 1)
			echo on tetanus # {j + 1}
			setclock 0 {accurate_dt}
			for (i = 0 ; i < npulses ; i = i + 1)
				setfield /control/stimctl NMDA_ampl \
					{getfield /control/NMDA_ampl value}
				setfield /control/stimctl non_NMDA_ampl \
					{getfield /control/non_NMDA_ampl value}
				// step // 1 step only, so that we cleanly
					// increment the activation of the channels	
				step {pulsewidth} -t
	
				setfield /control/stimctl NMDA_ampl 0
				setfield /control/stimctl non_NMDA_ampl 0
				step {inter_pulse_width} -t
			end
			if ({j + 1} < ntetani)
				setclock 0 {intertrial_dt}
				step {tet_spacing - tet_duration} -t
			end
		end
	end
	setclock 0 {intertrial_dt}
	step {post_tet_time} -t
	echo sim done, saving plots in {getfield /control/filename value}.*
	save_plot "/graphs/Conc/spinehead" "Ca_conc"
	save_plot "/graphs/Vm/spinehead" "Vm"
	save_plot "/graphs/Stim/NMDA_Gk" "NMDA_Gk"
	save_plot "/graphs/Stim/non_NMDA_Gk" "non_NMDA_Gk"
end


create xform /control [0,400,500,400]
create xbutton /control/start [,,33%,] -script do_step -offbg green
create xbutton /control/stop [0:last,0,34%,] -script stop -offbg yellow
create xbutton /control/reset [0:last,0,33%,] -script reset -offbg red
create xdialog /control/runtime [0,0:reset,50%,] -value 2.0 \
	-script "setfield /graphs/# xmax <v>"
create xdialog /control/currtime [0:last,0:reset,50%,] -value 0.0
create xdialog /control/dendinject [0,0:currtime,50%,] -value 0.0
	addmsg /control/dendinject /cell/CA1/apical# INJECT fvalue
create xdialog /control/somainject [0:last,0:currtime,50%,] -value 0.0
	addmsg /control/somainject /cell/CA1/soma INJECT fvalue

create xlabel /control/stimctl -label "Stimulation control" -bg cyan
addfield /control/stimctl NMDA_ampl
addfield /control/stimctl non_NMDA_ampl
setfield /control/stimctl NMDA_ampl 0
setfield /control/stimctl non_NMDA_ampl 0
addmsg /control/stimctl /cell/#/#[]/NMDA ACTIVATION NMDA_ampl
addmsg /control/stimctl /cell/#/#[]/non_NMDA ACTIVATION non_NMDA_ampl
create xdialog /control/ntetani [0,0:stimctl,33%,] -value {NTETANI}
create xdialog /control/tet_spacing [0:last,0:stimctl,34%,] -value {TET_SPACING}
create xdialog /control/npulses [0:last,0:stimctl,33%,] -value {NPULSES}
create xdialog /control/pulsefreq [0,0:ntetani,50%,] -value {PULSEFREQ}
create xdialog /control/pulsewidth [0:last,0:ntetani,50%,] \
	-value 0.00005 // Can't go any shorter due to clock. It is a bit
				// strange to have such short times anyway.
create xdialog /control/NMDA_ampl [0,0:pulsefreq,50%,] -value {NMDA_AMPL}
create xdialog /control/non_NMDA_ampl [0:last,0:pulsefreq,50%,] \
	-value {non_NMDA_AMPL}
create xdialog /control/pre_tet_time [0,0:NMDA_ampl,50%,30] \
	-value {PRE_TET_TIME}
create xdialog /control/post_tet_time [0:last,0:NMDA_ampl,50%,30] \
	-value {POST_TET_TIME}
create xbutton /control/do_stim \
	-label "Start stimulus train" -script do_stim

create xlabel /control/spacer -label "" -bg cyan
create xtoggle /control/disable_cell -onlabel "cell disabled" \
	-offlabel "cell enabled" \
	-script "disable.d /form/draw/cell; enable.u /form/draw/cell; resched"
create xdialog /control/filename [,,70%,30] -value {FILENAME}
create xbutton /control/Save_Ca_plot [0:last,0:disable_cell,30%,30] \
	-script "save_Ca_plot"
create xbutton /control/quit -script quit
addaction /control/currtime PROCESS update_time
useclock /control/currtime 3

create xform /graphs [500,0,500,850]
create xgraph /graphs/Vm [0,0,100%,50%] \
	-xmax 2 -ymin -0.065 -ymax 0.05 -yoffset 0.0
create xplot /graphs/Vm/soma 
create xplot /graphs/Vm/apical
create xplot /graphs/Vm/basal
create xplot /graphs/Vm/spinehead
create xplot /graphs/Vm/spinebase
addmsg /cell/CA1/soma /graphs/Vm/soma PLOT Vm *soma *red
addmsg /cell/CA1/apical_20 /graphs/Vm/apical PLOT Vm *apical *green
addmsg /cell/CA1/basal_8 /graphs/Vm/basal PLOT Vm *basal *blue
addmsg /cell/CA1/head1 /graphs/Vm/spinehead PLOT Vm *spinehead *orange
addmsg /cell/CA1/den0 /graphs/Vm/spinebase PLOT Vm *spinebase *white

create xgraph /graphs/Conc [0,0:last,100%,30%] \
	-xmax 2 -ymin 0.0 -ymax 0.01 -yoffset 0.01
// create xplot /graphs/Conc/apical
create xplot /graphs/Conc/spinehead
// addmsg /cell/CA1/apical_18/Ca_conc /graphs/Conc/apical PLOT Ca *apical *green
addmsg /cell/CA1/head1/Ca_pool /graphs/Conc/spinehead PLOT Co *spinehead *orange
create asc_file /out
setfield /out flush 0 leave_open 1 filename {FILENAME}.asc
useclock /out 1
addmsg /cell/CA1/head1/Ca_pool out SAVE Co

create xgraph /graphs/Stim [0,0:last,100%,20%] \
	-xmax 2 -ymin 0.0 -ymax 1e-10 -yoffset 0
addmsg /cell/CA1/head1/NMDA /graphs/Stim PLOT Gk *NMDA_Gk *red
addmsg /cell/CA1/head1/non_NMDA /graphs/Stim PLOT Gk *non_NMDA_Gk *blue
addmsg /cell/CA1/head1/pratio /graphs/Stim PLOT output *pratio *green
/*
addmsg /control/stimctl /graphs/Stim PLOT NMDA_ampl *NMDA_stim *red
addmsg /control/stimctl /graphs/Stim PLOT non_NMDA_ampl *nonNMDA_stim *blue
*/

useclock /graphs/## 3
useclock /graphs/Conc/spinehead 1
useclock /graphs/Vm/spinehead 1
useclock /graphs/Stim/NMDA_Gk 1
useclock /graphs/Stim/non_NMDA_Gk 1
useclock /graphs/Stim/pratio 1


call /cell/CA1/solve SETUP

setmethod /cell/CA1/solve 11

xshow /form
xshow /control
xshow /graphs

reset

do_stim
